@@ -2,12 +2,19 @@ import logging
 
 from django.conf import settings as django_settings
 from django.contrib.auth import authenticate, get_user_model, login, logout
-from django.http import Http404, HttpResponseRedirect
+from django.http import Http404, HttpResponse, HttpResponseRedirect
 from django.utils.decorators import method_decorator
 from django.views.decorators.cache import never_cache
 from django.views.generic import TemplateView
 from django.views.generic.base import RedirectView
 
+try:
+    from django.utils.http import url_has_allowed_host_and_scheme as safe_url
+except ImportError:
+    from django.utils.http import is_safe_url as safe_url
+
+from django.views.decorators.csrf import csrf_protect
+
 from . import settings
 from .forms import (
     LoginForm, SignupForm, SignupFormEmailOnly, SignupFormFull,
@@ -21,6 +28,7 @@ User = get_user_model()
 log = logging.getLogger(__name__)
 
 
+@method_decorator(csrf_protect, name='dispatch')
 class Login(TemplateView):
     template_name = settings.LOGIN_TEMPLATE_NAME
 
@@ -43,9 +51,11 @@ class Login(TemplateView):
         if not settings.REQUIRE_SIGNUP:
             get_or_create_user(email)
 
-        next_url = request.GET.get('next', '')
+        redirect_url = self.login_redirect_url(request.GET.get('next', ''))
         try:
-            magiclink = create_magiclink(email, request, redirect_url=next_url)
+            magiclink = create_magiclink(
+                email, request, redirect_url=redirect_url
+            )
         except MagicLinkError as e:
             form.add_error('email', str(e))
             context['login_form'] = form
@@ -61,6 +71,20 @@ class Login(TemplateView):
             log.info(f'Cookie {cookie_name} set for {email}')
         return response
 
+    def login_redirect_url(self, next_url) -> str:
+        redirect_url = ''
+        allowed_hosts = django_settings.ALLOWED_HOSTS
+        if '*' in allowed_hosts:
+            allowed_hosts = self.request.get_host()
+        url_is_safe = safe_url(
+            url=next_url,
+            allowed_hosts=allowed_hosts,
+            require_https=self.request.is_secure(),
+        )
+        if url_is_safe:
+            redirect_url = next_url
+        return redirect_url
+
 
 class LoginSent(TemplateView):
     template_name = settings.LOGIN_SENT_TEMPLATE_NAME
@@ -111,6 +135,7 @@ class LoginVerify(TemplateView):
         return response
 
 
+@method_decorator(csrf_protect, name='dispatch')
 class Signup(TemplateView):
     template_name = settings.SIGNUP_TEMPLATE_NAME
 
@@ -131,7 +156,10 @@ class Signup(TemplateView):
             'SignupFormFull',
         ]
         forms = __import__('magiclink.forms', fromlist=from_list)
-        SignupForm = getattr(forms, form_name)
+        try:
+            SignupForm = getattr(forms, form_name)
+        except AttributeError:
+            return HttpResponseRedirect(self.request.path_info)
 
         form = SignupForm(request.POST)
         if not form.is_valid():
